/*
* Konnect - a network tool for connecting to Internet
*
* Copyright (C) 2004  Fredrik Edemar
*                     f_edemar@linux.se
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*/

#include "konnect.h"
#include <math.h>

#include <kaction.h>
#include <kapp.h>
#include <kdebug.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kled.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kstdaction.h>
#include <kstatusbar.h>
#include <ktoolbar.h>

#include <qpushbutton.h>
#include <qtextstream.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qtimer.h>

#include "konnectsettings.h"

Konnect::Konnect() :
                DCOPObject("Konnect"),
		KMainWindow(),
		inne("green"),
		ute("red")
{	
	m_dlg = new KonnectWidgetDlg(this);
	setCentralWidget( m_dlg );

	// Setup the system tray
        connect(m_dlg->log_in, SIGNAL( clicked() ), this, SLOT( log_in() ) );
        connect(m_dlg->log_out, SIGNAL( clicked() ), this, SLOT (log_out() ) );
	connect(m_dlg->cmdCancel, SIGNAL( clicked() ), this, SLOT(closeConnection() ) );
        m_tray = new KSystemTray( this);
	KIconLoader *loader = KGlobal::iconLoader();
	icon_out = loader->loadIcon("offline", KIcon::User);
	icon_in = loader->loadIcon("online", KIcon::User);
	m_tray->setPixmap(icon_out);
	m_tray->show();
	
	m_settingsDialog = 0;
	m_showmsgbox = true;
	m_timer = 0;
	updateConfiguration();
 	setupActions();
	statusBar()->message(i18n("Disconnected"));
	
	if ( KonnectSettings::logInAutomatic() )
		log_in();
	else
		QTimer::singleShot( 0, this, SLOT(test_connection()) ); //don't do the connections test if we should log in directly
}

Konnect::~Konnect()
{
}

void Konnect::test_connection()
{
	mySocket = new QSocket();
	connect(mySocket, SIGNAL(connected()),this,SLOT(connectiontest_succeeded() ) );
	QTimer::singleShot( 5000, mySocket, SLOT(deleteLater()) );
	if (m_connectionType==2)
		mySocket->connectToHost("www.tiscali.se",80);
	else
 		mySocket->connectToHost("www.telia.com",80);
}

void Konnect::connectiontest_succeeded()
{
	mySocket->close();
	mySocket->deleteLater();
	m_dlg->status->setColor(inne);
	statusBar()->message(i18n("Connected"));
	m_tray->setPixmap(icon_in);
}

void Konnect::setupActions()
{
    KStdAction::quit(this, SLOT(slotQuit()), actionCollection(),"quit");
    KStdAction::showToolbar(this, SLOT(toggleToolBar()), actionCollection());
    KStdAction::showStatusbar(this, SLOT(toggleStatusBar()), actionCollection());
    KStdAction::preferences(this, SLOT(optionsPreferences()), actionCollection());
    KAction *mnu_logga_in = new KAction( i18n("Log &in"), "connect_creating",0, this, SLOT(log_in()), actionCollection(), "log_in" );
    KAction *mnu_logga_ut = new KAction( i18n("Log &out"), "connect_no", 0, this, SLOT(log_out()), actionCollection(), "log_out" );
    mnu_logga_in->plug(m_tray->contextMenu(),1);
    mnu_logga_ut->plug(m_tray->contextMenu(),2);
    
    createGUI();
    statusBar()->show();
}

void Konnect::slotQuit()
{
	doClosingThings();
	kapp->quit();
}

void Konnect::toggleToolBar()
{
  if(toolBar()->isVisible())
    toolBar()->hide();
  else
    toolBar()->show();
}

void Konnect::toggleStatusBar()
{
  if(statusBar()->isVisible())
    statusBar()->hide();
  else
    statusBar()->show();
}

void Konnect::log_in()
{
	m_dlg->cmdCancel->setEnabled(true);
        qWarning("Loggar in");// as: " + user);
	statusBar()->message(i18n("Logging in..." ));
	http.setHost(m_loginServer);
        http.get("/sd/init",0);
	http.closeConnection();

        connect(&http,SIGNAL(done( bool ) ), this, SLOT( loginDone ( bool ) ) );
	http.get("/sd/login?username="+user+"&password="+password+"&submitForm=Logga+in",&buffer);
        http.closeConnection();
}

void Konnect::loginDone(bool error )
{

        if ( error)
                qWarning("Error");
        buffer.close();
        buffer.open(IO_ReadOnly); //mste pnnas i readonly-lge
        QTextStream stream(&buffer);

        QString resultat("");
        while ( !stream.atEnd())
                resultat.append( stream.readLine() );
        if (!resultat.contains("newPane()") ) {
                statusBar()->message(i18n("Wrong user name or password")); //Felaktigt anvndarnamn/lsenord
		m_dlg->status->setColor(ute);
		m_tray->setPixmap(icon_out);
	}
        else {
                statusBar()->message(i18n("Connected"));
                m_dlg->status->setColor(inne);
		m_tray->setPixmap(icon_in);
        }

        buffer.close();
	disconnect(&http,SIGNAL(done( bool ) ), this, SLOT( loginDone ( bool ) ) );
	m_dlg->cmdCancel->setEnabled(false);

}

void Konnect::log_out()
{
	m_dlg->cmdCancel->setEnabled(true);
        qWarning("Loggar ut...");
	statusBar()->message(i18n("Logging out..." ));
	http.setHost(m_loginServer);
        connect(&http,SIGNAL(done( bool ) ), this, SLOT( logoutDone ( bool ) ) );
        http.get("/sd/logout",&buffer);
        http.closeConnection();
}

void Konnect::logoutDone(bool)
{
        buffer.close();
        buffer.open(IO_ReadOnly); //mste pnnas i readonly-lge
        QTextStream stream(&buffer);
        QString resultat;
	QString rad;
	
	while ( rad = stream.readLine() )
	{
		if ( rad.contains("<title>",false) && rad.contains("Error",false) )
		{
			buffer.close();
			disconnect(&http,SIGNAL(done( bool ) ), this, SLOT( logoutDone ( bool ) ) );
			m_dlg->cmdCancel->setEnabled(false);
			m_dlg->status->setColor(ute);
			statusBar()->message(i18n("Couln't log out"));
			return;
		}
		else if ( rad.contains("Sessionstid",false) )
		{
			int const startpos = rad.find("</B>",false)+4;
			for (uint i=startpos; i < rad.length();i++)
				if (rad.at(i)=='<')
				{
					resultat.append(i18n("Sessionstime: %1\n").arg(rad.mid(startpos,i-startpos).stripWhiteSpace()+"\n"));
					kdDebug() << "Sessionstime:" << rad << endl;
				}
		}
		else if ( rad.contains("var bytes_out",false) )
		{
			int const startpos = rad.find("=",false)+1;
			int const endpos=rad.length()-1;
			resultat.append(i18n("Received megabytes: %1\n").arg(QString::number(round(100 * rad.mid(startpos,endpos-startpos).simplifyWhiteSpace().toDouble() / 1048576.0) / 100.0)));
			kdDebug() << "var bytes_out:" << rad << endl;
			
		}
		else if ( rad.contains("var bytes_in",false) )
		{
			int const startpos = rad.find("=",false)+1;
			int const endpos=rad.length()-1;
			resultat.append(i18n("Sent megabytes: %1\n").arg(QString::number(round(100 * rad.mid(startpos,endpos-startpos).simplifyWhiteSpace().toDouble() / 1048576.0) / 100.0)));
			kdDebug() << "var bytes_in:" << rad << endl;
			
		}
	}
        buffer.close();
	disconnect(&http,SIGNAL(done( bool ) ), this, SLOT( logoutDone ( bool ) ) );
	m_dlg->cmdCancel->setEnabled(false);
	m_dlg->status->setColor(ute);
	statusBar()->message(i18n( "Disconnected"));
	m_tray->setPixmap(icon_out);
	if ( m_showmsgbox && !resultat.isEmpty())
		KMessageBox::information(this,resultat);
}

void Konnect::closeEvent( QCloseEvent *e )
{
	const QObject* caller = sender();
	while (m_tray && caller)
	{
		if (caller == m_tray) //om det r Systray som vill stnga av
			break; // avslutar loopen och caller r !=0
		caller = caller->parent();
	}
	if (!caller && !kapp->sessionSaving() )
	{
		hide(); //gmmer fnstret
		e->ignore();
		return;
	}
	
	doClosingThings();
	e->accept(); //avslutar programmet
}

void Konnect::doClosingThings()
{
	if ( KonnectSettings::logOutAtExit() )
	{
		m_showmsgbox=false;
		log_out();
		//kdDebug() << "Loggar ut................" << endl;
		m_showmsgbox=true;
	}
	if ( KonnectSettings::askAutostart() && !kapp->sessionSaving())
	{
		int const autoStart = KMessageBox::questionYesNoCancel( 0L, i18n("Should Konnect start automatically\nwhen you login?"), i18n("Automatically Start Konnect?") );
		KConfig *config = KGlobal::config();
		config->setGroup("General");
		if ( autoStart == KMessageBox::Yes )
			config->writeEntry("AutoStart", true);
		else if ( autoStart == KMessageBox::No)
			config->writeEntry("AutoStart", false);
	}
}

void Konnect::updateConfiguration()
{
	user = KonnectSettings::user();
	password = KonnectSettings::password();
	m_connectionType = KonnectSettings::connectionType();
	switch (m_connectionType)
	{
		case 3: //custom login server
			m_loginServer = KonnectSettings::loginServer();
			break;
		case 2: //Tiscali
			m_loginServer = "valo1.tiscali.se";
			break;
		case 1: //com hem
			m_loginServer = "login1.comhem.se";
			break;
		default: //Telia ADSL
			m_loginServer = "login1.telia.com";
	}
	
	if ( KonnectSettings::autoLogin() )
	{
		if ( !m_timer)
		{
			m_timer = new QTimer(this);
			connect( m_timer, SIGNAL( timeout() ), this, SLOT(log_in() ));
		}
		m_timer->start( KonnectSettings::autoInterval()*1000*60, false);
	}
	else if (m_timer)
	{
		delete m_timer;
		m_timer = 0;
	}
	
}

void Konnect::optionsPreferences()
{
	if (!m_settingsDialog)
	{
		m_settingsDialog = new KConfigDialog( this, "settings", KonnectSettings::self() );
		m_connectionSettings = new KConnectionSettings();
		m_generalSettings = new GeneralSettings(0, "General");
		m_intervalSettings = new IntervalSettings(0, "Interval");
		m_settingsDialog->addPage( m_generalSettings, i18n("General Settings"), "package_settings" );
		m_settingsDialog->addPage( m_connectionSettings, i18n("Connection settings"), "connect_established" );
		m_settingsDialog->addPage( m_intervalSettings, i18n("Interval settings"),NULL );
		connect( m_settingsDialog, SIGNAL(settingsChanged()), this, SLOT(updateConfiguration()) );
	}
	if ( KConfigDialog::showDialog( "settings" ) )
		return;
	m_generalSettings->show();
}

void Konnect::closeConnection()
{
	http.stop();
	m_dlg->cmdCancel->setEnabled(false);
	statusBar()->message(i18n("Stopped" ));
}

bool Konnect::isConnected()
{
	return m_dlg->status->color() == inne; //green == connected
}

#include "konnect.moc"
